﻿# 1_prepare_users.ps1 - Creates a file with current MailStore user names.
#                       That file can be used in the 2_update_users.ps1 script.
#
# Version 1.0: initial version
#
# Copyright (c) 2017 - 2022 MailStore Software GmbH
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# https://help.mailstore.com/en/server/PowerShell_API_Wrapper_Tutorial

# API Wrapper in same directory is used, nothing to configure here
Import-Module (Join-Path $PSScriptRoot "MS.PS.Lib.psd1") -ErrorAction Stop

######################################################
# Adjust these values according to your installation #
######################################################

# Username of an MailStore SPE Admin, default is "admin"
$username = "admin"

# If password is empty, an interactive prompt will ask for it
$password = ''

# The hostname of the MailStore Management Server, default is "localhost"
$hostname = "localhost"

# TCP Port of the Management API, API must be enabled in the MailStore SPE Configuration, default is 8474
$port = 8474

# Set this to $true when untrusted certificates are used, default is $false
$acceptAllCertificates = $false

# Instance ID of the instance to work on
$instanceID = "instanceID"

# File where the user list will be stored
$filename = Join-Path $env:HOMEDRIVE (Join-Path $env:HOMEPATH ("SPE-" + $instanceID + "-users.txt"))

############### FUNCTION CAN BE ADJUSTED TO MODIFY ARCHIVE I/O ###############

function Prepare-Output($archives) {
    return ($archives | Sort-Object | ForEach-Object { ($_ + "=" + $_) })
}

################### DO NOT CHANGE ANYTHING BELOW THIS LINE ###################

if ($password -ceq ''){
    $password = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR((Read-Host -AsSecureString -Prompt "Enter MailStore Password for $username")))
}

function Has-Succeeded($result) {
    if ($result.statusCode -ne "succeeded") {
        Write-Host ("Status  " + $result.statusCode)
        Write-Host ("Message " + $result.error.message)
        Write-Host ("Details " + $result.error.details)
        return $false
    }
    return $true
}

function My-Exit() {
    if (-not (Test-Path variable:global:psISE)) {
        Read-Host -Prompt "Press [Enter] to exit script"
    }
    exit
}

Write-Host "Creating API client"
$msapiclient = New-MSApiClient -Username $username -Password $password -ManagementServer $hostname -Port $port -IgnoreInvalidSSLCerts:$acceptAllCertificates

$archive_admin_enabled_result = Invoke-MSApiCall $msapiclient "GetArchiveAdminEnabled" @{"instanceID" = $instanceID}

if (-not (Has-Succeeded($archive_admin_enabled_result))) {
    Write-Host "Error while fetching archive admin configuration."
    My-Exit
}

if (-not ($archive_admin_enabled_result.result.value)) {
    Write-Host "Archive Admin Access is disabled, but has to be enabled. Aborting script execution."
    My-Exit
}

Write-Host "Fetching MailStore Users and Archives"
$mailstore_archives_result = Invoke-MSApiCall $msapiclient "GetChildFolders" @{"instanceID" = $instanceID; "maxLevels" = 1}

if (Has-Succeeded($mailstore_archives_result)){
    $mailstore_archives = $mailstore_archives_result.result.childFolders.name
}
else {
    My-Exit
}

Write-Host ("Writing Users and Archives to file " + $filename)
Prepare-Output($mailstore_archives) | Out-File $filename

Write-Host ("Opening file " + $filename)
&explorer.exe (Split-Path -Path $filename)
Start-Process -FilePath $filename

My-Exit
