﻿# 3_prepare_folders.ps1 - Creates a file with current MailStore folder names.
#                         That file can be used in the 4_update_folders.ps1 script.
#
# Version 1.1: CAS variables renamed to match script 4
#              pkv3 value not passed to CmdClient anymore when value is empty
# Version 1.0: initial version
#
# Copyright (c) 2017 - 2022 MailStore Software GmbH
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# https://help.mailstore.com/en/server/PowerShell_API_Wrapper_Tutorial

######################################################
# Adjust these values according to your installation #
######################################################

# Install the "E-mail Archive Client" and adjust the path to the MailStoreCmd.exe
$MailStoreCmd = "C:\Program Files (x86)\E-mail Archive Client\MailStoreCmd.exe"

# Username of an MailStore SPE Instance administrator, default is "admin", must not the global SPE admin!
$instanceUsername = "admin"

# Password of the instance administrator, if password is empty, an interactive prompt will ask for it
$instancePassword = ''

# The hostname of the MailStore SPE Client Access Server (CAS), default is "localhost"
$casHostname = "localhost"

# TCP Port of the Client Access, default is 443
$casPort = 443

# When using a certificate on the CAS that is not trusted, the certificate thumbprint must be entered here
$casHttpThumbprint = ''

# Instance ID of the instance to work on
$instanceID = "instanceID"

# File where the folder list will be stored
$filename = Join-Path $env:HOMEDRIVE (Join-Path $env:HOMEPATH ("SPE-" + $instanceID + "-folders.txt"))

############### FUNCTIONS CAN BE ADJUSTED TO MODIFY FOLDER I/O ###############

function Check-Folder($folder) {
    return $folder.fullName.Split("/")[1].StartsWith("Exchange")
}

function Prepare-Output($folderFullNames) {
    return ($folderFullNames | ForEach-Object { ($_ + "=" + $_) })
}

################### DO NOT CHANGE ANYTHING BELOW THIS LINE ###################

function My-Exit() {
    if (-not (Test-Path variable:global:psISE)) {
        Read-Host -Prompt "Press [Enter] to exit script"
    }
    exit
}

if (-not (Test-Path $MailStoreCmd -PathType Leaf)){
    Write-Host ($MailStoreCmd + " does not exist.")
    My-Exit
}

if ($instancePassword -ceq ''){
    $instancePassword = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR((Read-Host -AsSecureString -Prompt "Enter MailStore Instance Password for $instanceUsername ($instanceID)")))
}

# CmdClient ("command", "parameters")
function CmdClient([array]$command) {
    if (-not $command) {
        Write-Host "Error: No Command"
        My-Exit
    }

    $url = "https://{0}:{1}/{2}" -f $casHostname, $casPort, $instanceID
    
    if ($casHttpThumbprint -ceq ''){
        $cmdout = &$MailStoreCmd --h=$url --u=$instanceUsername --p=$instancePassword --nologo -c $command
    } else {
        $cmdout = &$MailStoreCmd --h=$url --u=$instanceUsername --p=$instancePassword --pkv3=$casHttpThumbprint --nologo -c $command
    }

    if($?) {
       #MailStoreCmd.exe succeeded
       $jsonresult = $cmdout -replace '(.*)\.\.\.', ""| Where-Object { $_ -ne "" }  #Strip connecting... disconnecting... installing...
       try {
           $result = ConvertFrom-Json (-join $jsonresult) -ErrorAction Stop;
       }
       catch {
           $result = $jsonresult
           Write-Host $jsonresult
       }
    }
    else {
        Write-Host "MailStoreCmd.exe failed"
        Write-Host ($cmdout -replace '(.*)\.\.\.', ""| Where-Object { $_ -ne "" })
        My-Exit
    }
    #Write-Host $result
    return $result
}

Write-Host "Fetching folder hierarchy."
$get_folders = CmdClient("GetChildFolders", "--maxLevels=2")

$rename_folders = [System.Collections.ArrayList]@()

foreach ($folder in $get_folders.childFolders) {
    foreach ($subfolder in $folder.childFolders) {
        if (Check-Folder($subfolder)) {
            $rename_folders.Add($subfolder.fullName) | Out-Null       
        }
    }
}

if (-not $rename_folders) {
    Write-Host "No folders to rename found."
    Write-Host
    Write-Host "If this is unexpected, check the following:"
    Write-Host "* 'Archive Access by Admins' has to be enabled in the instance's compliance settings"
    Write-Host "* no folder matches the criteria of the Check-Folder function of this script"
    Write-Host "* there are no subfolders"
    My-Exit
}

Write-Host ("Writing folders to file " + $filename)
Prepare-Output($rename_folders) | Out-File $filename

Write-Host ("Opening file " + $filename)
&explorer.exe (Split-Path -Path $filename)
Start-Process -FilePath $filename

My-Exit
