﻿# 6_update_archives_after_delete.ps1 - Reads the file created in 5_prepare_archives_after_delete.ps1.
#                                      Renames all archives found in the file.
#
# Version 1.1: check for upper case letters in target user name added
#              check for write-protected archive stores added
#              variables renamed
# Version 1.0: initial version
#
# Copyright (c) 2017 - 2022 MailStore Software GmbH
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# https://help.mailstore.com/en/server/PowerShell_API_Wrapper_Tutorial

# API Wrapper in same directory is used, nothing to configure here
Import-Module (Join-Path $PSScriptRoot "MS.PS.Lib.psd1") -ErrorAction Stop

######################################################
# Adjust these values according to your installation #
######################################################

# Username of an MailStore SPE admin, default is "admin"
$username = "admin"

# If password is empty, an interactive prompt will ask for it
$password = ''

# The hostname of the MailStore Management Server, default is "localhost"
$hostname = "localhost"

# TCP Port of the Management API, API must be enabled in the MailStore SPE Configuration, default is 8474
$port = 8474

# Set this to $true when untrusted certificates are used, default is $false
$acceptAllCertificates = $false

# Instance ID of the instance to work on
$instanceID = "instanceID"

# File where the archives list is stored
$filename = Join-Path $env:HOMEDRIVE (Join-Path $env:HOMEPATH ("SPE-" + $instanceID + "-archives.txt"))

################### DO NOT CHANGE ANYTHING BELOW THIS LINE ###################

if ($password -ceq ''){
    $password = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR((Read-Host -AsSecureString -Prompt "Enter MailStore Password for $username")))
}

function Has-Succeeded($result) {
    if ($result.statusCode -ne "succeeded") {
        Write-Host ("Status  " + $result.statusCode)
        Write-Host ("Message " + $result.error.message)
        Write-Host ("Details " + $result.error.details)
        return $false
    }
    return $true
}

function My-Exit() {
    if (-not (Test-Path variable:global:psISE)) {
        Read-Host -Prompt "Press [Enter] to exit script"
    }
    exit
}

if (-not (Test-Path $filename -PathType Leaf)){
    Write-Host ($filename + " does not exist.")
    My-Exit
}

Write-Host "Creating API client"
$msapiclient = New-MSApiClient -Username $username -Password $password -ManagementServer $hostname -Port $port -IgnoreInvalidSSLCerts:$acceptAllCertificates

$archive_admin_enabled_result = Invoke-MSApiCall $msapiclient "GetArchiveAdminEnabled" @{"instanceID" = $instanceID}

if (-not (Has-Succeeded($archive_admin_enabled_result))) {
    Write-Host "Error while fetching archive admin configuration."
    My-Exit
}

if (-not ($archive_admin_enabled_result.result.value)) {
    Write-Host "Archive Admin Access is disabled, but has to be enabled. Aborting script execution."
    My-Exit
}

Write-Host "Checking for write-protected archive stores"
$archive_stores_result = Invoke-MSApiCall $msapiclient "GetStores" @{"instanceID" = $instanceID}

if (-not (Has-Succeeded($archive_stores_result))) {
    My-Exit
}

if ($archive_stores_result.result.requestedState -match "writeProtected") {
    foreach ($archive_store in $archive_stores_result.result){
        if ($archive_store.requestedState -ceq "writeProtected") {
            Write-Host ("Archive store with ID: " + $archive_store.id + " Name: '" + $archive_store.name + "' is write protected")
        }
    }
    Write-Host ("At least one write-protected archive store has been found. Script aborting.")
    My-Exit
}

Write-Host ("Reading archives file " + $filename)
foreach ($line in (Get-Content $filename))
{
    # Skip lines starting with # and empty lines
    if ($line.StartsWith("#") -or -not $line) {
        Write-Host ("Skipping line '" + $line + "'")
        continue
    }
    
    $archive_name_parts = $line.Split("=")

    
    # skip archives which contain at least one equals character
    if ($archive_name_parts.count -gt 2) {
		Write-Host ("Source archive name or target archive name contains at least one equals character (=). Line: {0}. Skipping." -f $line)
		continue
	}

    $source_archive = $archive_name_parts[0]
    $target_archive = $archive_name_parts[1]

    # skip archives without source
    if (-not $source_archive){
        Write-Host ("Source archive name of '" + $target_archive + "' is empty. Skipping archive.")
        continue
    }

    # skip archives without target
    if (-not $target_archive){
        Write-Host ("Target archive name of '" + $source_archive + "' is empty. Skipping archive.")
        continue
    }

    # skip archives that do not change
    if ($source_archive -eq $target_archive){
        Write-Host ("Source archive name and target archive name must not match, but they do: " + $source_archive + " = " + $target_archive)
        continue
    }

    # convert archives where the target name contains upper case letters to lower case
    if ($target_archive -cne $target_archive.toLower()){
        Write-Host ("Target archive name must be all lower case but is'" + $target_archive + "'. Auto converted to lower case.")
        $target_archive =  $target_archive.toLower()
    }

    Write-Host ("Moving archive " + $source_archive + " to " + $target_archive)
    $rename_archive_result = Invoke-MSApiCall $msapiclient "MoveFolder" @{"instanceID" = $instanceID; "fromFolder" = $source_archive; "toFolder" = $target_archive}
    
    if (-not (Has-Succeeded($rename_archive_result))) {
        continue
    }
    # Write-Host rename_archive_result.statusCode
}

My-Exit
